/*
 * Copyright (c) 2005 INdT.
 * @author Andre Moreira Magalhaes <andre.magalhaes@indt.org.br> 
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "xmpp_chat.h"

#include "xmpp_chat_session_client.h"

#include <talk/p2p/base/helpers.h>
#include <talk/xmpp/xmppclient.h>

#include <sstream>

#include <unistd.h>

namespace {
    
const std::string STR_CHAT("chat");

}

XmppChat::XmppChat(XmppChatSessionClient *session_client, const buzz::Jid &remote_jid, bool incoming)
    : session_client_(session_client),
      remote_jid_(remote_jid),
      support_notifications_(false),
      first_message_(true),
      established_(incoming),
      established_cb_(NULL),
      established_cb_data_(NULL),
      message_received_cb_(NULL),
      message_received_cb_data_(NULL),
      delivery_error_cb_(NULL),
      delivery_error_cb_data_(NULL)
{
}

XmppChat::~XmppChat()
{
}
   
void XmppChat::SendMessage(const std::string &subject,
                           const std::string &body,
                           ChatState chat_state)
{
    XmppMessage message;
           
    /* we should always send chat state active if its the first message */
    crit_.Enter();
    if (support_notifications_ || !established_) {
        if (!first_message_) {
            if (established_) {
                message.chat_state = chat_state;
            }
            else {
                message.chat_state = CHAT_STATE_NONE;
            }
        }
        else {
            first_message_ = false;
            message.chat_state = CHAT_STATE_ACTIVE;
        }
    }
    /* do not send empty messages */
    if (subject.empty() && body.empty() && message.chat_state == CHAT_STATE_NONE) {
        crit_.Leave();
        return;
    }
    crit_.Leave();
 
    std::string id_str;
    std::stringstream st;
    st << cricket::CreateRandomId();
    st >> id_str;
    message.id = id_str;

    message.from = session_client_->GetClient()->jid().Str();
    crit_.Enter();
    message.to = remote_jid_.Str();
    crit_.Leave();
    
    message.type = STR_CHAT;
    message.subject = subject;
    message.body = body;
    // TODO fill thread field
        
    session_client_->SendStanza(session_client_->TranslateMessage(&message));
}

void XmppChat::SetRemoteJid(const buzz::Jid &jid) 
{   
    crit_.Enter();
    if (established_) {
        crit_.Leave();
        return;
    }
    
    established_ = true;
    remote_jid_ = jid; 
    crit_.Leave();
    
    if (established_cb_) {
        established_cb_((xmpp_chat_t) this, established_cb_data_);
    }
}
 
void XmppChat::OnMessage(const XmppMessage &message)
{
    crit_.Enter();
    if (!support_notifications_ && message.chat_state != CHAT_STATE_NONE) {
        support_notifications_ = true;
    }
    /* this is a outgoing chat and we received the first message */
    if (!established_) {
        established_ = true;
    }
    crit_.Leave();
    
    if (message_received_cb_) {
        message_received_cb_((xmpp_chat_t) this, message_received_cb_data_, 
            (xmpp_message_t) &message);
    }
}

void XmppChat::OnError(const XmppMessage &message, 
                       int error_code)
{    
    if (delivery_error_cb_) {
        delivery_error_cb_((xmpp_chat_t) this, 
                delivery_error_cb_data_, (xmpp_message_t) &message, error_code);
    }
}
